// Hippo GTK interface.

#include "hippo.h"

extern "C" {
#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>
#include <glib.h>
}

// The GTK interface.
interface_gtk *gtkif;

// Widgets
GtkTooltips *tooltips;
GtkWidget *window, *stopbutton, *shufflebutton, *buttons_box, *all_box,
  *scroll, *filelist=NULL, *statusbar, *sortbutton, *prevbutton,
  *nextbutton, *allbutton, *fileselector=NULL;
GtkAdjustment *vadj;
GtkAccelGroup *accel_group;

interface_gtk::interface_gtk(songlist *newsl) : interface(newsl) {
  pl = new playlist(newsl);
  newsl->select_all(pl);

  gtkif = this;
  last_song = -1;
}

interface_gtk::~interface_gtk() {
  delete pl;
}

/* Handle the "delete_event" event. */
gint handle_delete_event(GtkWidget *widget, GdkEvent *event, gpointer data) {

  return(FALSE); /* Close the window, as FALSE */
}

/* Handle the "destroy" event. */
void handle_destroy(GtkWidget *widget, gpointer data) {

  /* Then quit the interface. */
  gtk_main_quit();
}

// Called when the list is double-clicked.
void listpick_handler(GtkWidget *gtklist, gint pos, gint column, 
		      GdkEventButton *event, gpointer data) {

  // Select and play the song.
  gtkif->pl->select(pos);
  gtkif->pl->play();
}

// Button callbacks.
void next_handler(GtkWidget *widget, gpointer data) { 
  gtkif->pl->play_next(); 
}
void prev_handler(GtkWidget *widget, gpointer data) { 
  gtkif->pl->play_previous(); 
}
void stop_handler(GtkWidget *widget, gpointer data) { 
  gtkif->pl->stop(); 
}
void shuffle_handler(GtkWidget *widget, gpointer data) { 
  gtkif->pl->shuffle(); gtkif->fill_list(); 
}
void sort_handler(GtkWidget *widget, gpointer data) { 
  gtkif->pl->sort(playlist::by_name); gtkif->fill_list(); 
}
void all_handler(GtkWidget *widget, gpointer data) { 
  gtkif->sl->select_all(gtkif->pl); gtkif->fill_list();
}
void search_handler(GtkWidget *widget, gpointer data) { 
  char *key = "macy";
  gtkif->sl->select_match(gtkif->pl, key);
  gtkif->fill_list();
}

// Fill the listview with the items in the playlist.
void interface_gtk::fill_list() {
  unsigned int i;

  // Clear the listview.
  gtk_clist_clear(GTK_CLIST(filelist));

  // Add the items to the list.
  unsigned int s = pl->size();
  for(i=0; i<s; i++) {
    gchar *row[1];
    row[0] = (char *)pl->get_song(i).name.c_str();
    gtk_clist_append(GTK_CLIST(filelist), row);
  }
}

void interface_gtk::create(int& argc, char**& argv) {

  // Initialise GTK.
  gtk_init(&argc, &argv);

  /* Create the acceleration group. */
  accel_group = gtk_accel_group_new();

  /* Create the tooltips object. */
  tooltips = gtk_tooltips_new();

  /* Standard GTK initialisation. */
  window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title(GTK_WINDOW(window), "XHippo");
  gtk_window_set_policy(GTK_WINDOW(window), TRUE, TRUE, TRUE);
  gtk_signal_connect(GTK_OBJECT(window), "delete_event",
		     GTK_SIGNAL_FUNC(handle_delete_event), NULL);
  gtk_signal_connect(GTK_OBJECT(window), "destroy",
		     GTK_SIGNAL_FUNC(handle_destroy), NULL);
  //gtk_signal_connect(GTK_OBJECT(window), "configure_event",
  //		     GTK_SIGNAL_FUNC(handle_configure), NULL);

  /* Create the container for everything. */
  all_box = gtk_vbox_new(FALSE, 0);
  gtk_container_add(GTK_CONTAINER(window), all_box);
  gtk_widget_set_name(all_box, "all_box");

  /* Create the buttons container. */
  buttons_box = gtk_hbox_new(FALSE, 0);
  gtk_box_pack_start(GTK_BOX(all_box), buttons_box, FALSE, FALSE, 0);
  gtk_widget_set_name(buttons_box, "buttons_box");

  prevbutton = gtk_button_new_with_label("Prev");
  gtk_signal_connect(GTK_OBJECT(prevbutton), "clicked",
		     GTK_SIGNAL_FUNC(prev_handler), NULL);
  gtk_box_pack_start(GTK_BOX(buttons_box), prevbutton, TRUE, TRUE, 0);
  gtk_widget_set_name(prevbutton, "prevbutton");
  gtk_widget_show(prevbutton);

  nextbutton = gtk_button_new_with_label("Next");
  gtk_signal_connect(GTK_OBJECT(nextbutton), "clicked",
		     GTK_SIGNAL_FUNC(next_handler), NULL);
  gtk_box_pack_start(GTK_BOX(buttons_box), nextbutton, TRUE, TRUE, 0);
  gtk_widget_set_name(nextbutton, "nextbutton");
  gtk_widget_show(nextbutton);

  stopbutton = gtk_button_new_with_label("Stop");
  gtk_signal_connect(GTK_OBJECT(stopbutton), "clicked",
		     GTK_SIGNAL_FUNC(stop_handler), NULL);
  gtk_box_pack_start(GTK_BOX(buttons_box), stopbutton, TRUE, TRUE, 0);
  gtk_widget_set_name(stopbutton, "stopbutton");
  gtk_widget_show(stopbutton);

  shufflebutton = gtk_button_new_with_label("Shuffle");
  gtk_signal_connect(GTK_OBJECT(shufflebutton), "clicked",
		     GTK_SIGNAL_FUNC(shuffle_handler), NULL);
  gtk_box_pack_start(GTK_BOX(buttons_box), shufflebutton, TRUE, TRUE, 0);
  gtk_widget_set_name(shufflebutton, "shufflebutton");
  gtk_widget_show(shufflebutton);

  sortbutton = gtk_button_new_with_label("Sort");
  gtk_signal_connect(GTK_OBJECT(sortbutton), "clicked",
		     GTK_SIGNAL_FUNC(sort_handler), NULL);
  gtk_box_pack_start(GTK_BOX(buttons_box), sortbutton, TRUE, TRUE, 0);
  gtk_widget_set_name(sortbutton, "sortbutton");
  gtk_widget_show(sortbutton);

  allbutton = gtk_button_new_with_label("All");
  gtk_signal_connect(GTK_OBJECT(allbutton), "clicked",
		     GTK_SIGNAL_FUNC(all_handler), NULL);
  gtk_box_pack_start(GTK_BOX(buttons_box), allbutton, TRUE, TRUE, 0);
  gtk_widget_set_name(allbutton, "allbutton");
  gtk_widget_show(allbutton);

  /* Show the buttons box. */
  gtk_widget_show(buttons_box);

  /* Create the scroll that the list lives in. */
  scroll = gtk_scrolled_window_new(NULL, NULL);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scroll), 
				 GTK_POLICY_NEVER, GTK_POLICY_ALWAYS);
  gtk_box_pack_start(GTK_BOX(all_box), scroll, TRUE, TRUE, 0);
  gtk_widget_set_name(scroll, "scroll");
  vadj = gtk_scrolled_window_get_vadjustment(GTK_SCROLLED_WINDOW(scroll));

  /* Create the list. */
  // Create a new list.
  filelist = gtk_clist_new(1);

  // Set the list properties.
  gtk_clist_set_selection_mode(GTK_CLIST(filelist), GTK_SELECTION_SINGLE);
  gtk_clist_column_titles_hide(GTK_CLIST(filelist));
  gtk_clist_set_shadow_type(GTK_CLIST(filelist), GTK_SHADOW_ETCHED_IN);
  gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scroll),
					filelist);
  gtk_widget_set_name(filelist, "filelist");

  // Connect the handler.
  gtk_signal_connect(GTK_OBJECT(filelist), "select_row",
		     GTK_SIGNAL_FUNC(listpick_handler), NULL);

  /* Show the list */
  gtk_widget_show(filelist);

  /* Show the scroll. */
  gtk_widget_show(scroll);

  /* Show the everything box. */
  gtk_widget_show(all_box);

  /* Attach the acceleration group to the window. */
  gtk_accel_group_attach(accel_group, GTK_OBJECT(window));

  // Show the window.
  gtk_widget_show(window);
}

void interface_gtk::main() {

  // FIXME
  pl->set_continuous(true);

  // Fill the listview.
  fill_list();

  // Enter the GTK event loop.
  gtk_main();

  // Stop playing.
  pl->stop();
}

void interface_gtk::song_changed(int index) {

  // Avoid getting stuck in an endless loop.
  if (index == last_song) return;
  last_song = index;

  // FIXME: This should probably not use GTK functions, as it's
  // being called from a signal handler.
  //gtk_adjustment_set_value(GTK_ADJUSTMENT(vadj),
  //			   (vadj->lower)+((vadj->upper)-(vadj->lower))
  //			   *index/(pl->count()));
  if (!gtk_clist_row_is_visible(GTK_CLIST(filelist), index)) {
    gtk_clist_moveto(GTK_CLIST(filelist), index, 0, 0.5, 0.0);
  }
  gtk_clist_select_row(GTK_CLIST(filelist), index, 0);
}

