// Hippo playlist implementation.

#include "hippo.h"

extern "C" {
#include <time.h>
}

// Constructor.
playlist::playlist(songlist *newsl) : vector<int>() {
	// Seed the RNG using the current time.
	rng = subtractive_rng(time(NULL));

	sl = newsl;
	continuous = false;
	playing = false;
}

// Reset the position to the start.
void playlist::reset() {
	pos = begin();
}

// Play the next song.
int playlist::play_next() {
	++pos;
	if (pos == end()) pos = begin();
	return play();
}

// Play the previous song.
int playlist::play_previous() {
	--pos;
	if (pos == begin() - 1) pos = end() - 1;
	return play();
}

// Select a numbered song.
void playlist::select(int num) {
  pos = &((*this)[num]);
}

// Play the selected song.
int playlist::play() {
	if (!size()) return -1;

	playing = true;
	sl->play(*pos, this);

	iface->song_changed(pos - begin());
	
	return *pos;
}

// Stop playing.
void playlist::stop() {
	playing = false;
	sl->stop();
}

// Shuffle items.
void playlist::shuffle() {
	random_shuffle(begin(), end(), rng);
}

// Callback to indicate that the previous song finished.
void playlist::song_finished() {
	if (playing && continuous) play_next();
}

// Enable or disable continuous mode.
void playlist::set_continuous(bool newc) {
	continuous = newc;
}

// Return the song.
const song& playlist::get_song(int index) {
	return (*sl)[(*this)[index]];
}

// Class for sorting.
// FIXME: It would make more sense for this to be in songlist.
class playlist_sorter {
public:
	enum playlist::sort_methods method;
	songlist *sl;

	playlist_sorter(enum playlist::sort_methods newmethod, songlist *newsl) {
		method = newmethod;
		sl = newsl;
	}
	
	bool operator()(int a, int b) {
		switch (method) {
			case playlist::by_name:
				return (*sl)[a].name < (*sl)[b].name;
			case playlist::by_filename:
				return (*sl)[a].filename < (*sl)[b].filename;
		}
		cerr << "sort method unknown";
		return false; // should never happen; if it does, STL will get terribly confused
	}
};

// Sort the playlist.
void playlist::sort(enum sort_methods method) {
	playlist_sorter sorter(method, sl);

	::sort(begin(), end(), sorter);
}
