// Hippo songlist implementation.

#include "hippo.h"

// Constructor.
songlist::songlist(playermap *newpm) : vector<song>() {
	pm = newpm;
}

// Make a playlist containing all the current songs.
void songlist::select_all(playlist *pl) {
	pl->clear();
	for (unsigned int i=0; i<size(); i++) pl->push_back(i);
	pl->reset();
}

// Make a playlist containing songs that match a string in their title.
void songlist::select_match(playlist *pl, const string& key) {
	pl->clear();
	for (unsigned int i=0; i<size(); i++) {
		if ((*this)[i].name.find(key) != string::npos) pl->push_back(i);
	}
	pl->reset();
}

// Add a song to the list.
void songlist::add(const string& filename) {
	song s;

	s.filename = filename;
	s.name = pm->song_name(filename);

	push_back(s);
}

// Read a playlist, skipping lines beginning with hashes (which are
// comments in .m3u playlists).
// FIXME: This should handle xhippo !include directives.
void songlist::read_list(const string& name) {
	ifstream f(name.c_str());
	char *buf;
	char **news = &buf;

	for (;;) {
		f.gets(news);
		string s(*news);
		// hmm, does this leak?
		if (f.eof()) break;
		if (s[0] != '#') add(s);
	}
}

// Begin playing a song.
void songlist::play(int index, playlist *caller) {
	string cmd = pm->play_command((*this)[index].filename);
	subprocess_start(cmd, caller);
}

// Stop playing.
void songlist::stop() {
	subprocess_stop();
}
